#include "cubewidget.h"

CubeWidget::CubeWidget(QWidget *parent)
    : QGLWidget(QGLFormat(QGL::SampleBuffers), parent)
{

}

QSize CubeWidget::minimumSizeHint() const
{
    return QSize(50, 50);
}

QSize CubeWidget::sizeHint() const
{
    return QSize(600, 400);
}

void CubeWidget::initializeGL ()
{
    glClearColor(0.0, 0.0, 0.0, 0.0);
    glEnable(GL_DEPTH_TEST);


    // enable default shading
    showFlat();

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    static GLfloat lightPosition[4] = { 0.5, 0.0, 2.0, 1.0 };
    glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
}

void CubeWidget::setMaterial(GLfloat *color )
{
    glMaterialfv(GL_FRONT,GL_DIFFUSE,color);
    glMaterialfv(GL_FRONT,GL_SPECULAR,color);
    glMaterialfv(GL_FRONT,GL_SHININESS,color);
}

void CubeWidget::paintGL ( )
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();
    glTranslatef(0.0f,0.0f,-3.0f);
    glRotated(45,1,0,0);
    glRotated(45,0,1,0);


    GLfloat red[] = {1.0,0.0,0.0};
    GLfloat green[] = {0.0,1.0,0.0};
    GLfloat blue[] = {0.0,0.0,1.0};
    GLfloat cyan[] = {0.0,1.0,1.0};
    GLfloat magenta[] = {1.0,0.0,1.0};
    GLfloat yellow[] = {1.0,1.0,0.0};

    glBegin(GL_QUADS);
    setMaterial(red);
    glVertex3f( 0.5f, 0.5f,-0.5f);
    glVertex3f(-0.5f, 0.5f,-0.5f);
    glVertex3f(-0.5f, 0.5f, 0.5f);
    glVertex3f( 0.5f, 0.5f, 0.5f);

    setMaterial(magenta);
    glVertex3f( 0.5f,-0.5f, 0.5f);
    glVertex3f(-0.5f,-0.5f, 0.5f);
    glVertex3f(-0.5f,-0.5f,-0.5f);
    glVertex3f( 0.5f,-0.5f,-0.5f);

    setMaterial(green);
    glVertex3f( 0.5f, 0.5f, 0.5f);
    glVertex3f(-0.5f, 0.5f, 0.5f);
    glVertex3f(-0.5f,-0.5f, 0.5f);
    glVertex3f( 0.5f,-0.5f, 0.5f);

    setMaterial(yellow);
    glVertex3f( 0.5f,-0.5f,-0.5f);
    glVertex3f(-0.5f,-0.5f,-0.5f);
    glVertex3f(-0.5f, 0.5f,-0.5f);
    glVertex3f( 0.5f, 0.5f,-0.5f);

    setMaterial(blue);
    glVertex3f(-0.5f, 0.5f, 0.5f);
    glVertex3f(-0.5f, 0.5f,-0.5f);
    glVertex3f(-0.5f,-0.5f,-0.5f);
    glVertex3f(-0.5f,-0.5f, 0.5f);

    setMaterial(cyan);
    glVertex3f( 0.5f, 0.5f,-0.5f);
    glVertex3f( 0.5f, 0.5f, 0.5f);
    glVertex3f( 0.5f,-0.5f, 0.5f);
    glVertex3f( 0.5f,-0.5f,-0.5f);
    glEnd();
}

void CubeWidget::resizeGL(int width , int height )
{
    glMatrixMode(GL_PROJECTION);

    glViewport(0, 0, width, height);

    glLoadIdentity();
    perspective(45.0,1.0*width/height,0.01,5.0);

    glMatrixMode(GL_MODELVIEW);
}

void CubeWidget::perspective(GLdouble fovy, GLdouble aspect, GLdouble zNear, GLdouble zFar)
{
    GLdouble xmin, xmax, ymin, ymax;

    ymax = zNear * tan( fovy * M_PI / 360.0 );
    ymin = -ymax;
    xmin = ymin * aspect;
    xmax = ymax * aspect;

    glFrustum( xmin, xmax, ymin, ymax, zNear, zFar );
}


void CubeWidget::showWireframe()
{

    qDebug("show Wireframe");
    glShadeModel(GL_FLAT);
    glDisable(GL_CULL_FACE);

     glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
     repaint();
}

void CubeWidget::showFlat()
{
    qDebug("show Flat");
    glShadeModel(GL_FLAT);
    glEnable(GL_CULL_FACE);
    glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
    repaint();
}

void CubeWidget::showGouraut()
{
    qDebug("show Gouraut");
    glShadeModel(GL_SMOOTH);
    glEnable(GL_CULL_FACE);
    glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
    repaint();
}

void CubeWidget::showPhong()
{
    qDebug("show Phong");
    showFlat();

}

void CubeWidget::setTessellation(int t)
{
    qDebug()<<"Tesselation" << t;
}

void CubeWidget::mousePressEvent(QMouseEvent *event ) {}
void CubeWidget::mouseMoveEvent(QMouseEvent *event ) {}
void CubeWidget::wheelEvent(QWheelEvent *event ) {}
